class Resource<T, E> {
  T? data;
  E? error;

  Resource({this.data, this.error});

  bool get hasData => data != null;

  void resolve(
      {required dynamic Function(T data) onSuccess,
      dynamic Function(T? data)? onLoading,
      dynamic Function(E error)? onFailure}) {
    if (this is Success) {
      onSuccess(data!);
    } else if (onLoading != null && this is Loading) {
      onLoading(data);
    } else if (onFailure != null && this is Failure) {
      onFailure(error!);
    }
  }
}

class Success<T, E> extends Resource<T, E> {
  Success(T data) : super(data: data);
}

class Loading<T, E> extends Resource<T, E> {
  Loading(T? data) : super(data: data);
}

class Failure<T, E> extends Resource<T, E> {
  Failure(
    E error, {
    T? data,
  }) : super(data: data, error: error);
}
